"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatPath_filter = formatPath_filter;
exports.hasPathParameter_filter = hasPathParameter_filter;
exports.upperCamelCase_filter = upperCamelCase_filter;
exports.lowerCamelCase_filter = lowerCamelCase_filter;
exports.lowerSnakeCase_filter = lowerSnakeCase_filter;
exports.replaceRegEx_filter = replaceRegEx_filter;
exports.await_filter = await_filter;
exports.jsf_filter = jsf_filter;
exports.permuteOptionalArgs_filter = permuteOptionalArgs_filter;
exports.resolveRef_meta_filter = resolveRef_meta_filter;
exports.jsonSchemaCombineAllOf_meta_filter = jsonSchemaCombineAllOf_meta_filter;
const deterministic_json_schema_faker_1 = require("deterministic-json-schema-faker");
const seedrandom_1 = __importDefault(require("seedrandom"));
function formatPath_filter(path) {
    const splitPath = path.split('/').slice(1);
    return splitPath
        .map(el => {
        if (el.startsWith('{') && el.endsWith('}'))
            el = '#';
        el = el.replace(/[{}]/gi, '');
        return el;
    })
        .join('_')
        .replace(/s_#/, '_')
        .replace(/#/, '');
}
function hasPathParameter_filter(path) {
    const res = path.parameters?.some(p => p.in === 'path');
    return res ? true : false;
}
function upperCamelCase_filter(data) {
    return data
        .replaceAll('-', '_')
        .split('_')
        .map(s => s.charAt(0).toUpperCase() + s.slice(1))
        .join('');
}
function lowerCamelCase_filter(data) {
    return data
        .replaceAll('-', '_')
        .split('_')
        .map((s, i) => {
        if (i === 0)
            return s.charAt(0).toLowerCase() + s.slice(1);
        return s.charAt(0).toUpperCase() + s.slice(1);
    })
        .join('');
}
function lowerSnakeCase_filter(data) {
    return data
        .replaceAll('-', '_')
        .split('_')
        .map(s => s.charAt(0).toLowerCase() + s.slice(1))
        .join('_');
}
function replaceRegEx_filter(data, regex, replace) {
    return data.replace(new RegExp(regex, 'g'), replace);
}
async function await_filter(promise, callback) {
    try {
        callback(null, await promise);
    }
    catch (err) {
        callback(err);
    }
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function setAdditionalProperties(schema) {
    if (schema.type === 'object') {
        schema.additionalProperties = false;
    }
    if (schema.properties) {
        for (const key of Object.keys(schema.properties)) {
            setAdditionalProperties(schema.properties[key]);
        }
    }
    if (schema.anyOf) {
        for (const s of schema.anyOf) {
            setAdditionalProperties(s);
        }
    }
    if (schema.allOf) {
        for (const s of schema.allOf) {
            setAdditionalProperties(s);
        }
    }
    if (schema.items) {
        setAdditionalProperties(schema.items);
    }
    return schema;
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function jsf_filter(schema) {
    schema = setAdditionalProperties(schema);
    deterministic_json_schema_faker_1.JSONSchemaFaker.option({ random: (0, seedrandom_1.default)(schema), useExamplesValue: true });
    return deterministic_json_schema_faker_1.JSONSchemaFaker.generate(schema);
}
function permuteOptionalArgs_filter(args) {
    let products = [[]];
    for (let i = 0; i < args.length; i++) {
        products = [...products.map(a => [...a, args[i]]), ...products];
    }
    return products;
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function resolveRef_meta_filter(base) {
    return (ref) => {
        const path = ref.replace('#/', '').split('/');
        let obj = base;
        for (const p of path) {
            obj = obj[p];
        }
        return obj;
    };
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function merge_intersect(propA, propB) {
    if (propA && propB) {
        return [...propA].filter(x => propB.has(x));
    }
    return propA ?? propB;
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function merge_union(propA, propB) {
    return [...new Set([...(propA ?? []), ...(propB ?? [])])];
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function merge_single(propA, propB) {
    if (propA && propB) {
        if (propA !== propB)
            throw new Error("can't merge");
        return propA;
    }
    return propA ?? propB;
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function jsonSchemaCombineAllOf_meta_filter(base) {
    const resolve = resolveRef_meta_filter(base);
    const jsonSchemaCombineAllOf_filter = (schema) => {
        const new_schema = {};
        for (const arg of schema.allOf ?? []) {
            let resolved_arg = '$ref' in arg ? resolve(arg.$ref) : arg;
            if (resolved_arg.allOf) {
                resolved_arg = jsonSchemaCombineAllOf_filter(resolved_arg);
            }
            if (resolved_arg.type !== 'object')
                throw new Error('can only combine object types ' + JSON.stringify(resolved_arg));
            for (const prop of Object.keys(resolved_arg.properties ?? {})) {
                if (new_schema.properties?.[prop]) {
                    const propA = new_schema.properties?.[prop];
                    const propB = resolved_arg.properties?.[prop];
                    const merged_prop = {
                        ...propA,
                        ...propB,
                        enum: merge_intersect(propA.enum, propB.enum),
                        const: merge_single(propA.const, propB.const),
                    };
                    new_schema.properties = {
                        ...new_schema.properties,
                        ...{ [prop]: merged_prop },
                    };
                }
                else {
                    new_schema.properties = {
                        ...new_schema.properties,
                        ...{ [prop]: resolved_arg.properties?.[prop] },
                    };
                }
            }
            if (resolved_arg.required) {
                new_schema.required = merge_union(new_schema.required, resolved_arg.required);
            }
            for (const key of Object.keys(resolved_arg)) {
                if (key !== 'properties' && key !== 'required') {
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any
                    new_schema[key] = merge_single(resolved_arg[key], 
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any
                    new_schema[key]);
                }
            }
        }
        return new_schema;
    };
    return jsonSchemaCombineAllOf_filter;
}
//# sourceMappingURL=filter.js.map